#!/usr/bin/env python3

from itertools import product
import csv
import highspy
import json
import sys


def solve(data, alphas):
    n = len(data["cost"])
    workers = range(n)
    patients = range(n)
    workers_patients = list(product(workers, patients))

    h = highspy.Highs()
    h.setOptionValue("log_to_console", False)

    # x[w,p] = 1 if worker w is assigned to patient p.
    x = {(w, p): h.addBinary(obj=data["cost"][w][p]) for w, p in workers_patients}

    # Each worker is assigned to one patient.
    h.addConstrs(sum(x[w, p] for p in patients) == 1 for w in workers)

    # Each patient is assigned one worker.
    h.addConstrs(sum(x[w, p] for w in workers) == 1 for p in patients)

    # Primary objective: minimize cost.
    h.setMinimize()
    h.solve()
    cost = h.getObjectiveValue()

    cost_expr = sum(data["cost"][w][p] * x[w, p] for w, p in workers_patients)
    affinity_expr = sum(data["affinity"][w][p] * x[w, p] for w, p in workers_patients)

    for alpha in alphas:
        # Secondary objective: maximize affinity.
        for (w, p), x_wp in x.items():
            h.changeColCost(x_wp.index, data["affinity"][w][p])

        # Constrain cost to be within alpha of maximum.
        cost_cons = h.addConstr(cost_expr <= (1 + alpha) * cost)

        h.setMaximize()
        h.solve()
        affinity = h.getObjectiveValue()

        # Re-optimize with original cost objective, constraining affinity.
        for (w, p), x_wp in x.items():
            h.changeColCost(x_wp.index, data["cost"][w][p])
        affinity_cons = h.addConstr(affinity_expr >= affinity)

        h.setMinimize()
        h.solve()

        yield alpha, h.getObjectiveValue(), affinity

        # Remove cost and affinity constraints for
        h.removeConstr(cost_cons)
        h.removeConstr(affinity_cons)


def main():
    data = json.load(sys.stdin)

    out = csv.writer(sys.stdout)
    out.writerow(["alpha", "cost", "affinity"])

    for alpha, cost, affinity in solve(data, (i / 20.0 for i in range(21))):
        out.writerow([alpha, cost, affinity])


if __name__ == "__main__":
    main()
